<?php

/* 
 *  Search hooks
 */
 
function listing_db_rewrite_sql($query, $primary_table, $primary_field, $args) {
  if ($query == '' && $primary_table == 'n' && $primary_field = 'nid' && empty($args)) {
    $excluded_types = variable_get('listing_excluded_search_types', array());
    if (!empty($excluded_types)) {
      $where = " n.type NOT IN ('". join("','", $excluded_types) ."') ";
      return array('where' => $where);
    }
  }
}
function listing_search($op = 'search') {
  if ('admin' == $op) {
     $form = array();
     $form['listing_excluded_search_types'] = array(
       '#type'           => 'select',
       '#multiple'       => TRUE,
       '#title'          => t('Exclude Node Types'),
       '#default_value'  => variable_get('listing_excluded_search_types', array()),
       '#options'        => node_get_types('names'),
       '#size'           => 9,
       '#description'    => t('Node types to exclude from search results.'),
     );
     return $form;
  }
}

 /**
 * Function simply returns an array of fields to search when given a node
 */
function listing_fields_to_search($node){
  $fields = array();
  $ignore_fields = array(
    'op','attach-url','nid','vid','status','type','created','comment','changed','promote','sticky','revision_timestamp','log','format','uid','name','picture','data','path','last_comment_timestamp','last_comment_name','comment_count','files','iid','taxonomy','readmore','content','links','additionaltags'
  );
  $search_fields = array('title', 'body', 'teaser');
  foreach($node as $key=>$value){
    if(in_array($key, $search_fields) || substr($key, 0, 5) =="field"){
      $fields[] = $key;
    }
  }
  return $fields;
}
/*
 *  Autotag functions 
 */
function listing_search_for_tags($fields, $terms, $node) {
  $terms_to_return = array();
  foreach($fields as $field){
    // Lets split the fields up by spaces, and search on each word in the term_data table.
    if(isset($node->$field)){
      $terms_found = listing_search_field($node->$field, $terms);
      if(count($terms)){
        $terms_to_return = array_merge($terms_to_return, $terms_found);
      }
    }
  }
  return $terms_to_return;
}
/**
 * Takes a field and term and checks for a match.  Field can be an array, or
 * a single string.  If an array, it recurses till it gets a string
 * 
 * Search algorithm:
 * 
 *  @param $field - text field content
 *  @param $names = array of term names
 
 *  @return array of term tids to save to database
 */
function listing_search_field($field, $terms){
  $terms_found=array();
  $pattern ='/^[a-zA-Z]$/u';
  if(is_array($field) || is_object($field)){
    // Field is an array (and likely to be an array of arrays).  Lets
    // recurse into it and check away
    foreach($field as $field_part=>$value){
      $terms_found = array_merge($terms_found,listing_search_field($value, $terms));
    }
  } else {
    // Only search if there is text
    if(trim($field)==''){return array();}
    // Loop through terms and check if found in text
    foreach($terms as $term){
      foreach($term->synonyms as $synonym) {
	  //if(strtolower($synonym)=='aba') drupal_set_message('aba found');
	  //drupal_set_message($synonym);
 	    $pos = strpos('z'.strtolower(strip_tags($field)), strtolower($synonym));
        if($pos!== FALSE){
		  $pos = $pos -1; // this is to compensate for the 'z' we added above which we added to get any fields that start with a synonym
		  // we need to make sure this is a word on its own and not a part word 
		  $text = strip_tags($field);
		  $first_char = FALSE;
		  $last_char = FALSE;
		  // At the start and end of a field there is no character before or after
		  if($pos>0) {
		    $start = $pos-1;
			$start_char = substr($text, $start, 1);
		  }
		  else $first_char = TRUE;
		  $end = $pos+strlen($synonym);
		  if($end == strlen(strip_tags($field))) $last_char = TRUE;
		  else $end_char = substr($text, $end, 1);
		  
		  $found = TRUE;
		  // if either start or end characters are a letter then we don't save this match
		  if(preg_match($pattern, $start_char) && !$first_char) {
		    drupal_set_message('char start '.$start_char);
			drupal_set_message(' synonym '.$synonym);
			$found = FALSE;
		  }
		  if(preg_match($pattern, $end_char) && !$last_char) {
		    drupal_set_message('char end '.$end_char);
			drupal_set_message(' synonym '.$synonym);
			$found = FALSE;
		  }
          if($found) $terms_found[] = array('tid'=>$term->tid,'vid'=>$term->vid);
		  drupal_set_message('word '.$synonym.' found');
        }

	  }
	}
  }
  return $terms_found;
}
function listing_process_autotags($terms_found, $node) {
      
   if(!variable_get('listing_autotag_only', 1)) {
      if(!isset($node->taxonomy)){
        $terms = array();
      } else {
        $terms = $node->taxonomy;
      }
      // Check to see whether or not we have objects
      $terms_are_objects = false;
      foreach($terms as $term){
        if(is_object($term)){
          $terms_are_objects = true;
	    }
        break;
      }
	}
	else $terms = array();
      foreach($terms_found as $tid){
	   
        if($terms_are_objects){
          $terms[$tid['tid']] = taxonomy_get_term($tid['tid']);
        } else {
            if(is_array($terms[$tid['vid']])){
              if(!isset($terms[$tid['vid']]) || !in_array($tid['tid'], $terms[$tid['vid']])){
                $terms[$tid['vid']][] = $tid['tid'];
              }
            } else {
              // If there isn't currently anything set
              if(is_string($terms[$tid['vid']]) && trim($terms[$tid['vid']]) == ''){
                $terms[$tid['vid']] = $tid['tid'];
              } else {
                // else combine
                $terms[$tid['vid']] = array($tid['tid'], $terms[$tid['vid']]);
              }
            }
        }
      }
	  
      if($terms_are_objects){
        taxonomy_node_save($node, array_keys($terms));
      } else {
	    if($node->nid>0) db_query("DELETE FROM {term_node} WHERE nid=%d", $node->nid);
	    
        taxonomy_node_save($node, array_unique(array_flatten(array($terms))));
      }
      return;
}
function array_flatten($array) {
  $new_array = array();
  foreach ($array as $child) {
    if (is_array($child) || is_object($child)) {
      $new_array = array_merge($new_array, array_flatten($child));
    } else {
      $new_array[] = $child;
    }
  }
  return $new_array;
}


